;----------------------------------------------------------------------------------------------------------
; Title			:	CountDown\1_Main.asm
;----------------------------------------------------------------------------------------------------------
; Funktion		:	Das Gert verbindet einen Verbraucher fr eine einstellbare Zeit mit dem 230 VAC Netz.
;				:	Geschaltet werden die Leitungen L und N mit 2 24 VDC Relais mit 16 A 230 VAC Kontakten.
;				:	Bedienung:
;				:	Beide Tasten drcken:	Die Dekade "Stunden x 10" beginnt zu blinken. Wert: 0
;				:	Taste 1:				Auf die nchste Dekade umschalten, welche dann blinkt.
;				:	Taste 2:				Wert in der blinkenden Dekade ndern.
;				:	Betrieb mit 8 Dekaden:	Maximal eingebbarer Wert: 23:59:59.99	[HH:MM:SS.ss]
;				:	Betrieb mit 6 Dekaden:	Maximal eingebbarer Wert: 23:59:59		[HH:MM:SS]
;				:	Taste 1:				Nach der Eingabe des Wertes in die letzte Dekade:
;				:							Die Relais werden eingeschaltet.
;				:							Die gewhlte Zeit wird mit 100 Hz zurckgezhlt.
;				:							Weil diese Frequenz aus der Quarzfrequenz gewonnen wird,
;				:							wird die Zeit sehr przise eingehalten.
;				:							Wenn 00:00:00.00 erreicht ist, werden die Relais ausgeschaltet
;				:							und ein kleiner Lautsprecher summt fr ca. 2 Sekunden.
;				:	Whrend des Rckzhl-Vorgangs werden fhrende Nullen ausgeblendet.
;				:	Das gilt nicht fr die Dekaden "Sekunden x 1", "Sekunden x 0.1" und "Sekunden x 0.01"
;				:	Weil die 7 Segment Anzeigen keinen Doppelpunkt haben, wird der Dezimalpunkt angezeigt.
;				:	Dezimalpunkt jeweils bei Dekade:	"Stunden  x 1"
;				:										"Minuten  x 1"
;				:										"Sekunden x 1" (nur bei Betrieb mit 8 Dekaden)
;				:	Wenn eine dieser Dekaden ausgeblendet wird, weil es sich um eine fhrende Null handelt,
;				:	dann wird auch der Dezimalpunkt dieser Dekade ausgeblendet.
;				:	
; Anzeige		:	IC ICM7218A. Das ist ein Treiber fr 7 Segment + DP LED-Anzeigen mit gemeinsamer Anode.
;				:	In dieser Version werden dafr 9 Output Pins des Mikrocontrollers bentigt.
;				:	Hinweis:
;				:	Das Datenblatt des ICM7218 fordert Pausen- bzw. Signal-Haltezeiten. 
;				:	Diese werden offenbar eingehalten, ohne dass sie programmiert werden mssen.
;				:
; Schaltung		:	Schaltplan "CountDown.spl7"
;				:
; Ports und		:	Ports			ATMEGA8A		ICM7218A
; ICM7218A		:
;				:	B.0	Output		 Pin 14			Pin 12 = ID 0  HEX 0 = 1	
;				:	B.1	Output		 Pin 15			Pin 11 = ID 1  HEX 1 = 2
;				:	B.2	Output		 Pin 16			Pin 13 = ID 2  HEX 2 = 4
;				:	B.3	Output		 Pin 17			Pin 14 = ID 3  HEX 3 = 8
;				:
;				:	B.4	Output		 Pin 18			Pin 08 = High: Input Not Loaded
;				:					 				Pin 08 = Low : Input	 Loaded
;				:
;				:	B.5	Output		 Pin 19			Pin 09 = High: Load Control Data on /WRITE LOW Pulse
;				:									Pin 09 = Low : Load Input   Data on /WRITE LOW Pulse
;				:
;				:	C.0	Output		 Pin 23			Pin 05 = High: Hexadecimal Decoding
;				:									Pin 05 = Low : Code B	   Decoding
;				:
;				:	C.1	Output		 Pin 24			Pin 06 = High: No Decode
;				:									Pin 06 = Low :    Decode
;				:
;				:	C.2	Output		 Pin 25			Pin	07 = High:	  Data Coming
;				:									Pin 07 = Low : No Data Coming + Decimal Point
;				:
;				:	unbenutzt						Pin 10 = High: NO SHUTDOWN
;				:									Pin 10 = Low :    SHUTDOWN
;				:
;				:
; Ports			:									Normalbetrieb			Debugbetrieb
;				:
;				:	C.3	Output		 Pin 26			900 Hz					900 Hz (LED Anzeige)
;				:	C.4	Output		 Pin 27			 						 10 Hz (LED Anzeige)
;				:	C.5	Output		 Pin 28			100 Hz					  1 Hz (LED Anzeige)
;				:
;				:
;				:	D.2	Input		 Pin  4			IC1 (74LS14) "onInt0" (Taste 2)
;				:	D.3	Input		 Pin  5			IC1 (74LS14) "onInt1" (Taste 1)
;				:
;				:	D.4	Input		 Pin  6			High: 100 Hz, Low: 1 Hz (zum Debuggen)
;				:
;				:	D.5	Input		 Pin 11			High: Betrieb mit 8 Dekaden
;				:									Dezimalpunkte hinter			Stunden  x 1
;				:																	Minuten  x 1
;				:																	Sekunden x 1
;				:
;				:									Low:  Betrieb mit 6 Dekaden, ohne	Sekunden x 0.10
;				:															 und ohne	Sekunden x 0.01
;				:
;				:										  Dezimalpunkte hinter			Stunden  x 1
;				:																		Minuten  x 1
;				:																	
;				:	D.6 Output		 Pin 12			Relais
;				:	D.7	Output		 Pin 13			Lautsprecher
;				:
;----------------------------------------------------------------------------------------------------------
; Prozessor		:	ATmega8
; Takt			:	3,6864 MHz
; Sprache		:	Assembler
; Datum			:	15.08.2012
; Version		:	1.0
; Autor			:	rodo38
;----------------------------------------------------------------------------------------------------------
 		.include "m8def.inc"
;----------------------------------------------------------------------------------------------------------
; Variable definieren

		.def	mCoun1	=	 R0
		.def	mCoun2	=	 R1
		.def	mCoun3	=	 R2
		.def	mCoun4	=	 R3
		.def	mCoun5	=	 R4
		.def	mCoun6	=	 R5
		.def	mCoun7	=	 R6
		.def	mCoun8	=	 R7
		.def	mTimr1	=	 R8
		.def	mTimr2	=	 R9
		.def	mTimr3	=	R10
		.def	mTimr4	=	R11
		.def	mTimr5	=	R12

		.def	Temp	=	R16
		.def	Time	=	R17
		.def	Contr0	=	R18	
		.def	Contr1	=	R19	
		.def	Number	=	R20
		.def	Help00	=	R21
		.def	Freq00	=	R22
		.def	Temp01	=	R23
		.def	Help01	=	R24

;----------------------------------------------------------------------------------------------------------
; Reset and Interrupt Vector				Beschreibung

Begin:	rjmp	Main						;  1 POWER ON RESET
		rjmp	onInt0						;  2 Int0-Interrupt
		rjmp	onInt1						;  3 Int1-Interrupt
		reti								;  4 TC2 Compare Match
		reti								;  5 TC2 Overflow
		reti								;  6 TC1 Capture
		reti								;  7 TC1 Compare Match A
		reti								;  8 TC1 Compare Match B
		reti								;  9 TC1 Overflow
		rjmp	onTc0						; 10 TC0 Overflow
		reti								; 11 SPI, STC Serial Transfer Complete
		reti								; 12 UART Rx complete
		reti								; 13 UART Data Register Empty
		reti								; 14 UART Tx complete
		reti								; 15 ADC Conversion Complete
		reti								; 16 EEPROM Ready
		reti								; 17 Analog Comparator
		reti								; 18 TWI (IC) Serial Interface
		reti								; 19 Store Program Memory Redy
		
;----------------------------------------------------------------------------------------------------------
; Start, Power ON, Reset

Main:
		ldi		Temp	,	LOW (RAMEND)	; fr  Stackpointer LOW
		out		SPL		,	Temp			; INIT Stackpointer LOW

		ldi		Temp	,	HIGH(RAMEND)	; fr  Stackpointer HIGH
		out		SPH		,	Temp			; INIT Stackpointer HIGH

		ldi		Temp	,	0b00001111		; Bit 2 = 1, Bit 3 = 1: steigende Flanke erzeugt Interrupt
		out		MCUCR	,	Temp			; 
											
		ldi		Temp	,	0b11000000		; Bit 6 = Int0 aktiv
		out		GICR	,	Temp			; Bit 7 = Int1 aktiv

		ldi		Temp	,	0b00000010		; Bit 1 = 1: Prescaler 8
		out		TCCR0	,	Temp			; 

		ldi		Temp	,	0b00000000		; Bit 0 = 1: Timer0 Overflow
		out		TIMSK	,	Temp			; Bit 0 = 0: Timer nicht aktiv

		ldi		Temp	,	0b00111111		; B.0 bis B.5 = Output
		out		DDRB	,	Temp			;
		ldi		Temp	,	0b00111111		; C.0 bis C.5 = Output
		out		DDRC	,	Temp			;
		ldi		Temp	,	0b11000000		; D.2 bis D.5 = Input
		out		DDRD	,	Temp			; D.6 und D.7 = Output
		ldi		Temp	,	0b00111100		; D.2 bis D.5 = Pullup
		out		PORTD	,	Temp			;
		ldi		Temp	,	0b00000000		; PORTB und PORTC: Alle Bits = 0
		out		PORTB	,	Temp			;
		out		PORTC	,	Temp			;
		
;----------------------------------------------------------------------------------------------------------
; Variable initialisieren

		ldi		Temp	,	0b00000000
		ldi		Time	,	0b00000000
		ldi		Contr0	,	0b00000000
		ldi		Contr1	,	0b00000000
		ldi		Number	,	0b00000000
		ldi		Help00	,	0b00000000
		ldi		Freq00	,	0b00000000
		ldi		Temp01	,	0b00000000
		ldi		Help01	,	0b00000000
		
		mov		mCoun1	,	Temp
		mov		mCoun2	,	Temp
		mov		mCoun3	,	Temp
		mov		mCoun4	,	Temp
		mov		mCoun5	,	Temp
		mov		mCoun6	,	Temp
		mov		mCoun7	,	Temp
		mov		mCoun8	,	Temp

		mov		mTimr1	,	Temp
		mov		mTimr2	,	Temp
		mov		mTimr3	,	Temp
		mov		mTimr4	,	Temp
		mov		mTimr5	,	Temp

;----------------------------------------------------------------------------------------------------------
; Interrupt erlauben

		sei

;----------------------------------------------------------------------------------------------------------
; Files includet

		.include "2_Select.asm"
		.include "3_Decades.asm"
		.include "4_Digits.asm"
		.include "5_Timer.asm"
		.include "6_Down.asm"

;---------1---------2---------3---------4---------5---------6---------7---------8---------9---------0------
